/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2014 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
*                                                                             *
******************************************************************************/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include <system.h>
#include <altera_avalon_pio_regs.h>
#include <stdio.h>
#include <mcapi.h>

/*
*********************************************************************************************************
*                                             LOCAL DEFINES
*********************************************************************************************************
*/
#define  MCAPI_TASK_PRIO		(4)
#define  TASK_STK_SIZE			(2048)
#define  REMOTE_NODE			(0)
#define  THIS_NODE			(1)
#define  F2SDRAM_FPGAPORT_IN_RST	(0)

/*
*********************************************************************************************************
*                                            LOCAL VARIABLES
*********************************************************************************************************
*/

static  OS_STK         task_stk[TASK_STK_SIZE];

/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  mcapi_task (void  *p_arg);
extern  void  startup (unsigned int local, unsigned int remote);
extern  void  demo (unsigned int node, int loop);
extern  void  disconnect (unsigned int local, unsigned int remote);
extern  void  cleanup (void);



/*
*********************************************************************************************************
*                                                main()
*
* Description : This is the standard entry point for C code.  It is assumed that your code will call
*               main() once you have performed all necessary initialization.
*
* Argument(s) : none
*
* Return(s)   : none
*
* Caller(s)   : Startup Code.
*
* Note(s)     : none.
*********************************************************************************************************
*/

int  main (void)
{
	OSTaskCreateExt(mcapi_task,
			NULL,
			(void *) &task_stk[TASK_STK_SIZE - 1],
			MCAPI_TASK_PRIO,
			MCAPI_TASK_PRIO,
			&task_stk[0],
			TASK_STK_SIZE,
			NULL,
			(OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR));

	OSStart();   /* Start multitasking (i.e. give control to uC/OS-II).  */


	while (1) {  /* Should not get here                                  */
	    ;
	}

	return 0;
}


/*
*********************************************************************************************************
*                                         mcapi_task()
*
* Description : MCAPI demo
*
* Argument(s) : none.
*
* Caller(s)   : main
*
* Return(s)   : none.
*
* Note(s)     : none.
*********************************************************************************************************
*/

void mcapi_task(void *p_arg)
{
    (void)p_arg;
    char c;

    printf("\nStart MCAPI demo for Nios-II uC/OS-II\n");

    while (1) {
	startup(THIS_NODE, REMOTE_NODE);
	demo(THIS_NODE, 1);
	disconnect(THIS_NODE, REMOTE_NODE);
	cleanup();
	printf ("Press R to restart demo, E to end the demo\n");

	do {
		c = getchar();
	} while(c != 'R' && c != 'E' && c != 'r' && c != 'e');

	if (c == 'E' || c == 'e')
		break;
    }
    printf("== MCAPI demo end ==\n");
}
